@@ -27,7 +27,6 @@ from gettext import gettext as _
 
 from gi.repository import Gdk, GObject, Gtk
 
-from gnomemusic import log
 from gnomemusic.player import PlayerPlaylist
 from gnomemusic.search import Search
 from gnomemusic.widgets.albumcover import AlbumCover
@@ -57,7 +56,6 @@ class SearchView(Gtk.Stack):
         ARTIST = 4
 
     search_state = GObject.Property(type=int, default=Search.State.NONE)
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
     state = GObject.Property(type=int, default=State.MAIN)
 
@@ -74,11 +72,7 @@ class SearchView(Gtk.Stack):
     _view_all_albums = Gtk.Template.Child()
     _view_all_artists = Gtk.Template.Child()
 
-    def __repr__(self):
-        return '<SearchView>'
-
-    @log
-    def __init__(self, application, player=None):
+    def __init__(self, application):
         """Initialize SearchView
 
         :param GtkApplication application: The Application object
@@ -134,7 +128,7 @@ class SearchView(Gtk.Stack):
             'selection-mode', self._window, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL)
 
-        self._album_widget = AlbumWidget(player, self)
+        self._album_widget = AlbumWidget(self._application)
         self._album_widget.bind_property(
             "selection-mode", self, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -144,7 +138,6 @@ class SearchView(Gtk.Stack):
         self._scrolled_artist_window = None
 
         self._search_mode_active = False
-        # self.connect("notify::search-state", self._on_search_state_changed)
 
     def _core_filter(self, coreitem, coremodel, nr_items):
         if coremodel.get_n_items() <= 5:
@@ -180,7 +173,7 @@ class SearchView(Gtk.Stack):
 
         # NOTE: Adding SYNC_CREATE here will trigger all the nested
         # models to be created. This will slow down initial start,
-        # but will improve initial 'selecte all' speed.
+        # but will improve initial 'select all' speed.
         album_widget.bind_property(
             "selected", corealbum, "selected",
             GObject.BindingFlags.BIDIRECTIONAL)
@@ -206,6 +199,11 @@ class SearchView(Gtk.Stack):
         nr_albums = self._album_model.get_n_items()
         self._view_all_albums.props.visible = (nr_albums > model.get_n_items())
 
+        def set_child_visible(child):
+            child.props.visible = True
+
+        self._album_flowbox.foreach(set_child_visible)
+
     def _on_artist_model_items_changed(self, model, position, removed, added):
         items_found = model.get_n_items() > 0
         self._artist_header.props.visible = items_found
@@ -216,6 +214,11 @@ class SearchView(Gtk.Stack):
         self._view_all_artists.props.visible = (
             nr_artists > model.get_n_items())
 
+        def set_child_visible(child):
+            child.props.visible = True
+
+        self._artist_flowbox.foreach(set_child_visible)
+
     def _on_model_items_changed(self, model, position, removed, added):
         items_found = model.get_n_items() > 0
         self._songs_header.props.visible = items_found
@@ -295,7 +298,7 @@ class SearchView(Gtk.Stack):
         if nb_children == 0:
             return
 
-        first_child = self._album_flowbox.get_child_at_index(0)
+        first_child = self._artist_flowbox.get_child_at_index(0)
         # FIXME: It looks like it is possible that the widget is not
         # yet created, resulting in a crash with first_child being
         # None.
@@ -427,21 +430,20 @@ class SearchView(Gtk.Stack):
             if value is True:
                 view.select_all()
             else:
-                view.select_none()
+                view.deselect_all()
         elif self.props.state == SearchView.State.ARTIST:
             view = self.get_visible_child().get_child().get_child()
             if value is True:
                 view.select_all()
             else:
-                view.select_none()
+                view.deselect_all()
 
     def select_all(self):
         self._select_all(True)
 
-    def unselect_all(self):
+    def deselect_all(self):
         self._select_all(False)
 
-    @log
     def _back_button_clicked(self, widget, data=None):
         if self.get_visible_child() == self._search_results:
             return
@@ -455,18 +457,10 @@ class SearchView(Gtk.Stack):
         self.props.state = SearchView.State.MAIN
         self._headerbar.props.state = HeaderBar.State.MAIN
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if not self.props.selection_mode:
-            self.unselect_all()
-
-    @log
-    def _on_search_state_changed(self, klass, param):
-        # If a search is triggered when selection mode is activated,
-        # reset the number of selected items.
-        if (self.props.selection_mode
-                and self.props.search_state != Search.State.NONE):
-            self.props.selected_items_count = 0
+        if (not self.props.selection_mode
+                and self.get_parent().get_visible_child() == self):
+            self.deselect_all()
 
     @GObject.Property(type=bool, default=False)
     def search_mode_active(self):
