@@ -1,11 +1,4 @@
-# Copyright (c) 2013 Eslam Mostafa <cseslam@gmail.com>
-# Copyright (c) 2013 Vadim Rutkovsky <vrutkovs@redhat.com>
-# Copyright (c) 2013 Sai Suman Prayaga <suman.sai14@gmail.com>
-# Copyright (c) 2013 Arnel A. Borja <kyoushuu@yahoo.com>
-# Copyright (c) 2013 Shivani Poddar <shivani.poddar92@gmail.com>
-# Copyright (c) 2013 Manish Sinha <manishsinha@ubuntu.com>
-# Copyright (c) 2013 Seif Lotfy <seif@gmail.com>
-# Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
+# Copyright 2019 The GNOME Music developers
 #
 # GNOME Music is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,185 +25,209 @@
 from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
-from gnomemusic import log
+from gnomemusic.gstplayer import Playback
 from gnomemusic.mediakeys import MediaKeys
-from gnomemusic.player import Player, RepeatMode
-from gnomemusic.query import Query
+from gnomemusic.player import RepeatMode
+from gnomemusic.search import Search
+from gnomemusic.trackerwrapper import TrackerState
 from gnomemusic.utils import View
 from gnomemusic.views.albumsview import AlbumsView
 from gnomemusic.views.artistsview import ArtistsView
 from gnomemusic.views.emptyview import EmptyView
 from gnomemusic.views.searchview import SearchView
 from gnomemusic.views.songsview import SongsView
-from gnomemusic.views.playlistview import PlaylistView
+from gnomemusic.views.playlistsview import PlaylistsView
 from gnomemusic.widgets.headerbar import HeaderBar
-from gnomemusic.widgets.notificationspopup import NotificationsPopup
+from gnomemusic.widgets.notificationspopup import NotificationsPopup  # noqa
 from gnomemusic.widgets.playertoolbar import PlayerToolbar
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
-from gnomemusic.widgets.searchbar import Searchbar
-from gnomemusic.widgets.selectiontoolbar import SelectionToolbar
+from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
+from gnomemusic.widgets.selectiontoolbar import SelectionToolbar  # noqa: F401
 from gnomemusic.windowplacement import WindowPlacement
-from gnomemusic.playlists import Playlists
-from gnomemusic.grilo import grilo
-
-import logging
-logger = logging.getLogger(__name__)
-
-playlists = Playlists.get_default()
 
 
+@Gtk.Template(resource_path="/org/gnome/Music/ui/Window.ui")
 class Window(Gtk.ApplicationWindow):
 
+    __gtype_name__ = "Window"
+
+    active_view = GObject.Property(type=GObject.GObject, default=None)
     selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
-    def __repr__(self):
-        return '<Window>'
+    notifications_popup = Gtk.Template.Child()
+    _box = Gtk.Template.Child()
+    _overlay = Gtk.Template.Child()
+    _selection_toolbar = Gtk.Template.Child()
+    _stack = Gtk.Template.Child()
 
-    @log
     def __init__(self, app):
+        """Initialize the main window.
+
+        :param Gtk.Application app: Application object
+        """
         super().__init__(application=app, title=_("Music"))
 
-        self._settings = Gio.Settings.new('org.gnome.Music')
+        # Hack
+        self._app = app
+
+        self._app._coreselection.bind_property(
+            "selected-items-count", self, "selected-items-count")
+
+        self._settings = app.props.settings
         self.add_action(self._settings.create_action('repeat'))
         select_all = Gio.SimpleAction.new('select_all', None)
         select_all.connect('activate', self._select_all)
         self.add_action(select_all)
-        select_none = Gio.SimpleAction.new('select_none', None)
-        select_none.connect('activate', self._select_none)
-        self.add_action(select_none)
+        deselect_all = Gio.SimpleAction.new('deselect_all', None)
+        deselect_all.connect('activate', self._deselect_all)
+        self.add_action(deselect_all)
 
         self.set_size_request(200, 100)
-        WindowPlacement(self, self._settings)
+        WindowPlacement(self)
 
-        self.prev_view = None
-        self.curr_view = None
+        self._current_view = None
+        self._view_before_search = None
+
+        self._player = app.props.player
 
-        self.notifications_popup = NotificationsPopup()
         self._setup_view()
 
         MediaKeys(self._player, self)
 
-        grilo.connect('changes-pending', self._on_changes_pending)
-
-    @log
-    def _on_changes_pending(self, data=None):
-        # FIXME: This is not working right.
-        def songs_available_cb(available):
-            view_count = len(self.views)
-            if (available
-                    and view_count == 1):
-                self._switch_to_player_view()
-            elif (not available
-                    and not self.props.selection_mode
-                    and view_count != 1):
-                self._stack.disconnect(self._on_notify_model_id)
-                self.disconnect(self._key_press_event_id)
-
-                for i in range(View.ALBUM, view_count):
-                    view = self.views.pop()
-                    view.destroy()
-
-                self._switch_to_empty_view()
-
-        grilo.songs_available(songs_available_cb)
-
-    @log
     def _setup_view(self):
-        self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self._headerbar = HeaderBar()
-        self._searchbar = Searchbar()
-        self._player = Player(self)
-        self._player_toolbar = PlayerToolbar(self._player, self)
-        selection_toolbar = SelectionToolbar()
-        self.views = [None] * len(View)
-        self._stack = Gtk.Stack(
-            transition_type=Gtk.StackTransitionType.CROSSFADE,
-            transition_duration=100,
-            homogeneous=False,
-            visible=True,
-            can_focus=False)
+        self._search = Search()
+        self._headerbar_stack = Gtk.Stack()
+        transition_type = Gtk.StackTransitionType.CROSSFADE
+        self._headerbar_stack.props.transition_type = transition_type
+        self._headerbar = HeaderBar(self._app)
+        self._search_headerbar = SearchHeaderBar(self._app)
+        self._search_headerbar.props.stack = self._stack
+        self._headerbar_stack.add_named(self._headerbar, "main")
+        self._headerbar_stack.add_named(self._search_headerbar, "search")
+        self._headerbar_stack.props.name = "search"
 
-        self._headerbar.bind_property(
-            "search-mode-enabled", self._searchbar, "search-mode-enabled",
-            GObject.BindingFlags.BIDIRECTIONAL |
+        self._search.bind_property(
+            "search-mode-active", self._headerbar, "search-mode-active",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "search-mode-active", self._search_headerbar, "search-mode-active",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "state", self._search_headerbar, "search-state",
             GObject.BindingFlags.SYNC_CREATE)
-        self._searchbar.props.stack = self._stack
+
+        self._search.connect(
+            "notify::search-mode-active", self._on_search_mode_changed)
+
+        self._player_toolbar = PlayerToolbar()
+        self._player_toolbar.props.player = self._player
+
         self._headerbar.connect(
             'back-button-clicked', self._switch_back_from_childview)
 
-        self.connect('notify::selection-mode', self._on_selection_mode_changed)
         self.bind_property(
             'selected-items-count', self._headerbar, 'selected-items-count')
         self.bind_property(
-            'selected-items-count', selection_toolbar, 'selected-items-count')
+            "selected-items-count", self._selection_toolbar,
+            "selected-items-count")
         self.bind_property(
             'selection-mode', self._headerbar, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
-            'selection-mode', selection_toolbar, 'visible',
-            GObject.BindingFlags.SYNC_CREATE)
+            "selected-items-count", self._search_headerbar,
+            "selected-items-count")
+        self.bind_property(
+            "selection-mode", self._search_headerbar, "selection-mode",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            "selection-mode", self._player_toolbar, "visible",
+            GObject.BindingFlags.INVERT_BOOLEAN)
+        self.bind_property(
+            "selection-mode", self._selection_toolbar, "visible")
+        self.connect("notify::selection-mode", self._on_selection_mode_changed)
+
+        self.views = [Gtk.Box()] * len(View)
         # Create only the empty view at startup
         # if no music, switch to empty view and hide stack
         # if some music is available, populate stack with mainviews,
         # show stack and set empty_view to empty_search_view
         self.views[View.EMPTY] = EmptyView()
+        self._stack.connect(
+            "notify::visible-child", self._on_stack_visible_child_changed)
         self._stack.add_named(self.views[View.EMPTY], "emptyview")
 
         # Add the 'background' styleclass so it properly hides the
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
-        self._overlay = Gtk.Overlay()
-        self._overlay.add(self._stack)
-        # FIXME: Need to find a proper way to do this.
-        self._overlay.add_overlay(self._searchbar._dropdown)
-        self._overlay.add_overlay(self.notifications_popup)
-        self.set_titlebar(self._headerbar)
-        self._box.pack_start(self._searchbar, False, False, 0)
-        self._box.pack_start(self._overlay, True, True, 0)
-        self._box.pack_start(self._player_toolbar, False, False, 0)
-        self._box.pack_start(selection_toolbar, False, False, 0)
-        self.add(self._box)
+        self._box.pack_end(self._player_toolbar, False, False, 0)
 
-        self._headerbar._search_button.connect(
-            'toggled', self._on_search_toggled)
-        selection_toolbar.connect(
+        self.set_titlebar(self._headerbar_stack)
+
+        self._selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
+        self._search.connect("notify::state", self._on_search_state_changed)
 
         self._headerbar.props.state = HeaderBar.State.MAIN
-        self._headerbar.show()
-        self._overlay.show()
-        self._player_toolbar.show_all()
-        self._box.show()
-        self.show()
+        self._headerbar_stack.show_all()
 
-        def songs_available_cb(available):
-            if available:
-                self._switch_to_player_view()
-            else:
-                self._switch_to_empty_view()
+        self._app.props.coremodel.connect(
+            "notify::songs-available", self._on_songs_available)
 
-        if Query().music_folder:
-            grilo.songs_available(songs_available_cb)
+        self._app.props.coremodel.props.grilo.connect(
+            "notify::tracker-available", self._on_tracker_available)
+
+        if self._app.props.coremodel.props.songs_available:
+            self._switch_to_player_view()
         else:
             self._switch_to_empty_view()
 
-    @log
     def _switch_to_empty_view(self):
         did_initial_state = self._settings.get_boolean('did-initial-state')
 
-        if not grilo.props.tracker_available:
-            self.views[View.EMPTY].props.state = EmptyView.State.NO_TRACKER
+        state = self._app.props.coremodel.props.grilo.props.tracker_available
+        empty_view = self.views[View.EMPTY]
+        if state == TrackerState.UNAVAILABLE:
+            empty_view.props.state = EmptyView.State.NO_TRACKER
+        elif state == TrackerState.OUTDATED:
+            empty_view.props.state = EmptyView.State.TRACKER_OUTDATED
         elif did_initial_state:
-            self.views[View.EMPTY].props.state = EmptyView.State.EMPTY
+            empty_view.props.state = EmptyView.State.EMPTY
         else:
-            self.views[View.EMPTY].props.state = EmptyView.State.INITIAL
+            # FIXME: On switch back this view does not show properly.
+            empty_view.props.state = EmptyView.State.INITIAL
 
         self._headerbar.props.state = HeaderBar.State.EMPTY
 
-    @log
+    def _on_search_mode_changed(self, search, value):
+        if self._search.props.search_mode_active:
+            self._headerbar_stack.set_visible_child_name("search")
+        else:
+            self._headerbar_stack.set_visible_child_name("main")
+
+    def _on_songs_available(self, klass, value):
+        if self._app.props.coremodel.props.songs_available:
+            self._switch_to_player_view()
+        else:
+            self._switch_to_empty_view()
+
+    def _on_stack_visible_child_changed(self, klass, value):
+        self.props.active_view = self._stack.props.visible_child
+
+    def _on_tracker_available(self, klass, value):
+        grilo = self._app.props.coremodel.props.grilo
+        new_state = grilo.props.tracker_available
+
+        if new_state != TrackerState.AVAILABLE:
+            self._switch_to_empty_view()
+
+        self._on_songs_available(None, None)
+
     def _switch_to_player_view(self):
         self._settings.set_boolean('did-initial-state', True)
         self._on_notify_model_id = self._stack.connect(
@@ -225,6 +242,8 @@ class Window(Gtk.ApplicationWindow):
         self._btn_ctrl.props.button = 8
         self._btn_ctrl.connect("pressed", self._on_back_button_pressed)
 
+        self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
+
         # FIXME: In case Grilo is already initialized before the views
         # get created, they never receive a 'ready' signal to trigger
         # population. To fix this another check was added to baseview
@@ -234,27 +253,19 @@ class Window(Gtk.ApplicationWindow):
         # to window._stack. For this to succeed, the stack needs to be
         # filled with something: Gtk.Box.
         # This is a bit of circular logic that needs to be fixed.
-        self.views[View.ALBUM] = Gtk.Box()
-        self.views[View.ARTIST] = Gtk.Box()
-        self.views[View.SONG] = Gtk.Box()
-        self.views[View.PLAYLIST] = Gtk.Box()
-        self.views[View.SEARCH] = Gtk.Box()
-
-        self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
         self._headerbar.props.state = HeaderBar.State.MAIN
         self._headerbar.props.stack = self._stack
-        self._searchbar.show()
 
-        self.views[View.ALBUM] = AlbumsView(self, self._player)
-        self.views[View.ARTIST] = ArtistsView(self, self._player)
-        self.views[View.SONG] = SongsView(self, self._player)
-        self.views[View.PLAYLIST] = PlaylistView(self, self._player)
-        self.views[View.SEARCH] = SearchView(self, self._player)
+        # All views are created together, so if the album view is
+        # already initialized, assume the rest are as well.
+        if not isinstance(self.views[View.ALBUM], Gtk.Box):
+            return
 
-        selectable_views = [View.ALBUM, View.ARTIST, View.SONG, View.SEARCH]
-        for view in selectable_views:
-            self.views[view].bind_property(
-                'selected-items-count', self, 'selected-items-count')
+        self.views[View.ALBUM] = AlbumsView(self._app)
+        self.views[View.ARTIST] = ArtistsView(self._app)
+        self.views[View.SONG] = SongsView(self._app)
+        self.views[View.PLAYLIST] = PlaylistsView(self._app)
+        self.views[View.SEARCH] = SearchView(self._app)
 
         # empty view has already been created in self._setup_view starting at
         # View.ALBUM
@@ -269,31 +280,27 @@ class Window(Gtk.ApplicationWindow):
         self._stack.set_visible_child(self.views[View.ALBUM])
 
         self.views[View.SEARCH].bind_property(
-            'search-state', self._searchbar, 'search-state')
+            "search-state", self._search, "state",
+            GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "search-mode-active", self.views[View.SEARCH],
+            "search-mode-active", GObject.BindingFlags.BIDIRECTIONAL)
+        self._search.bind_property(
+            "search-mode-active", self.views[View.ALBUM],
+            "search-mode-active", GObject.BindingFlags.SYNC_CREATE)
 
-    @log
     def _select_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
-        if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self._stack.get_visible_child()
-        else:
-            view = self._stack.get_visible_child().get_visible_child()
 
-        view.select_all()
+        self.props.active_view.select_all()
 
-    @log
-    def _select_none(self, action=None, param=None):
+    def _deselect_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
-        if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self._stack.get_visible_child()
-            view.unselect_all()
-        else:
-            view = self._stack.get_visible_child().get_visible_child()
-            view.select_none()
 
-    @log
+        self.props.active_view.deselect_all()
+
     def _on_key_press(self, widget, event):
         modifiers = event.get_state() & Gtk.accelerator_get_default_mod_mask()
         (_, keyval) = event.get_keyval()
@@ -311,7 +318,8 @@ class Window(Gtk.ApplicationWindow):
             if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
                     and self._headerbar.props.state != HeaderBar.State.SEARCH):
-                self._searchbar.toggle()
+                search_mode = self._search.props.search_mode_active
+                self._search.props.search_mode_active = not search_mode
             # Play / Pause on Ctrl + SPACE
             if keyval == Gdk.KEY_space:
                 self._player.play_pause()
@@ -321,8 +329,6 @@ class Window(Gtk.ApplicationWindow):
             # Play next on Ctrl + N
             if keyval == Gdk.KEY_n:
                 self._player.next()
-            if keyval == Gdk.KEY_q:
-                self.props.application.quit()
             # Toggle repeat on Ctrl + R
             if keyval == Gdk.KEY_r:
                 if self._player.props.repeat_mode == RepeatMode.SONG:
@@ -344,7 +350,7 @@ class Window(Gtk.ApplicationWindow):
         # Ctrl+Shift+<KEY>
         elif modifiers == shift_ctrl_mask:
             if keyval == Gdk.KEY_A:
-                self._select_none()
+                self._deselect_all()
         # Alt+<KEY>
         elif modifiers == mod1_mask:
             # Go back from child view on Alt + Left
@@ -374,7 +380,7 @@ class Window(Gtk.ApplicationWindow):
             if keyval == Gdk.KEY_AudioNext:
                 self._player.next()
 
-            child = self._stack.get_visible_child()
+            child = self.props.active_view
             if (keyval == Gdk.KEY_Delete
                     and child == self.views[View.PLAYLIST]):
                 self.views[View.PLAYLIST].remove_playlist()
@@ -382,38 +388,40 @@ class Window(Gtk.ApplicationWindow):
             if keyval == Gdk.KEY_Escape:
                 if self.props.selection_mode:
                     self.props.selection_mode = False
-                else:
-                    self._searchbar.reveal(False)
+                elif self._search.props.search_mode_active:
+                    self._search.props.search_mode_active = False
 
         # Open the search bar when typing printable chars.
         key_unic = Gdk.keyval_to_unicode(keyval)
-        if ((not self._searchbar.get_search_mode()
+        if ((not self._search.props.search_mode_active
                 and not keyval == Gdk.KEY_space)
                 and GLib.unichar_isprint(chr(key_unic))
                 and (modifiers == shift_mask
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
                 and self._headerbar.props.state != HeaderBar.State.SEARCH):
-            self._searchbar.reveal(True)
+            self._search.props.search_mode_active = True
 
-    @log
     def _on_back_button_pressed(self, gesture, n_press, x, y):
-        self._headerbar._on_back_button_clicked()
+        self._headerbar.emit('back-button-clicked')
 
-    @log
     def _notify_mode_disconnect(self, data=None):
         self._player.stop()
         self.notifications_popup.terminate_pending()
         self._stack.disconnect(self._on_notify_model_id)
 
-    @log
     def _on_notify_mode(self, stack, param):
-        self.prev_view = self.curr_view
-        self.curr_view = stack.get_visible_child()
+        previous_view = self._current_view
+        self._current_view = self.props.active_view
 
-        if (self.curr_view != self.views[View.SEARCH]
-                and self.curr_view != self.views[View.EMPTY]):
-            self._searchbar.reveal(False)
+        # Disable search mode when switching view
+        search_views = [self.views[View.EMPTY], self.views[View.SEARCH]]
+        if (self._current_view in search_views
+                and previous_view not in search_views):
+            self._view_before_search = previous_view
+        elif (self._current_view not in search_views
+                and self._search.props.search_mode_active is True):
+            self._search.props.search_mode_active = False
 
         # Disable the selection button for the EmptySearch and Playlist
         # view
@@ -421,16 +429,10 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.EMPTY],
             self.views[View.PLAYLIST]
         ]
-        allowed = self.curr_view not in no_selection_mode
+        allowed = self._current_view not in no_selection_mode
         self._headerbar.props.selection_mode_allowed = allowed
+        self._search_headerbar.props.selection_mode_allowed = allowed
 
-        # Disable renaming playlist if it was active when leaving
-        # Playlist view
-        if (self.prev_view == self.views[View.PLAYLIST]
-                and self.views[View.PLAYLIST].rename_active):
-            self.views[View.PLAYLIST].disable_rename_playlist()
-
-    @log
     def _toggle_view(self, view_enum):
         # TODO: The SEARCH state actually refers to the child state of
         # the search mode. This fixes the behaviour as needed, but is
@@ -441,27 +443,14 @@ class Window(Gtk.ApplicationWindow):
                 and not self._headerbar.props.state == HeaderBar.State.SEARCH):
             self._stack.set_visible_child(self.views[view_enum])
 
-    @log
-    def _on_search_toggled(self, button, data=None):
-        self._searchbar.reveal(
-            button.get_active(), self.curr_view != self.views[View.SEARCH])
-        if (not button.get_active()
-                and (self.curr_view == self.views[View.SEARCH]
-                    or self.curr_view == self.views[View.EMPTY])):
-            child = self.curr_view.get_visible_child()
-            if self._headerbar.props.state == HeaderBar.State.MAIN:
-                # We should get back to the view before the search
-                self._stack.set_visible_child(
-                    self.views[View.SEARCH].previous_view)
-            elif (self.views[View.SEARCH].previous_view == self.views[View.ALBUM]
-                    and child != self.curr_view._album_widget
-                    and child != self.curr_view._artist_albums_widget):
-                self._stack.set_visible_child(self.views[View.ALBUM])
+    def _on_search_state_changed(self, klass, param):
+        if (self._search.props.state != Search.State.NONE
+                or not self._view_before_search):
+            return
 
-            if self.props.selection_mode:
-                self.props.selection_mode = False
+        # Get back to the view before the search
+        self._stack.set_visible_child(self._view_before_search)
 
-    @log
     def _switch_back_from_childview(self, klass=None):
         if self.props.selection_mode:
             return
@@ -470,38 +459,31 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.ALBUM],
             self.views[View.SEARCH]
         ]
-        if self.curr_view in views_with_child:
-            self.curr_view._back_button_clicked(self.curr_view)
+        if self._current_view in views_with_child:
+            self._current_view._back_button_clicked(self._current_view)
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if self.props.selection_mode:
+        if (not self.props.selection_mode
+                and self._player.state == Playback.STOPPED):
             self._player_toolbar.hide()
-        elif self._player.props.playing:
-            self._player_toolbar.show()
-        if not self.props.selection_mode:
-            self._on_changes_pending()
 
-    @log
     def _on_add_to_playlist(self, widget):
-        if self._stack.get_visible_child() == self.views[View.PLAYLIST]:
+        if self.props.active_view == self.views[View.PLAYLIST]:
             return
 
-        def callback(selected_songs):
-            if len(selected_songs) < 1:
-                return
+        selected_songs = self._app._coreselection.props.selected_items
 
-            playlist_dialog = PlaylistDialog(
-                self, self.views[View.PLAYLIST].pls_todelete)
-            if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
-                playlists.add_to_playlist(
-                    playlist_dialog.get_selected(), selected_songs)
-            self.props.selection_mode = False
-            playlist_dialog.destroy()
+        if len(selected_songs) < 1:
+            return
 
-        self._stack.get_visible_child().get_selected_songs(callback)
+        playlist_dialog = PlaylistDialog(self)
+        if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
+            playlist = playlist_dialog.props.selected_playlist
+            playlist.add_songs(selected_songs)
+
+        self.props.selection_mode = False
+        playlist_dialog.destroy()
 
-    @log
     def set_player_visible(self, visible):
         """Set PlayWidget action visibility
 
